@@ -16,6 +16,11 @@ import abc
 import contextlib
 
 import six
+try:
+    # Python 3 no longer has thread module
+    import thread  # noqa
+except ImportError:
+    import threading as thread
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -66,8 +71,10 @@ class Transaction(object):
 
 @six.add_metaclass(abc.ABCMeta)
 class API(object):
-    def __init__(self):
-        self._nested_txn = None
+    def __init__(self, nested_transactions=True):
+        # Mapping between a (green)thread and its transaction.
+        self._nested_txns = nested_transactions
+        self._nested_txns_map = {}
 
     @abc.abstractmethod
     def create_transaction(self, check_error=False, log_errors=True, **kwargs):
@@ -82,26 +89,34 @@ class API(object):
         """
 
     @contextlib.contextmanager
-    def transaction(self, check_error=False, log_errors=True, **kwargs):
+    def transaction(self, check_error=False, log_errors=True, nested=True,
+                    **kwargs):
         """Create a transaction context.
 
         :param check_error: Allow the transaction to raise an exception?
         :type check_error:  bool
         :param log_errors:  Log an error if the transaction fails?
         :type log_errors:   bool
+        :param nested:      Allow nested transactions be merged into one txn
+        :type nested:       bool
         :returns: Either a new transaction or an existing one.
         :rtype: :class:`Transaction`
         """
-        if self._nested_txn:
-            yield self._nested_txn
-        else:
+        # ojbect() is unique, so if we are not nested, this will always result
+        # in a KeyError on lookup and so a unique Transaction
+        nested = nested and self._nested_txns
+        cur_thread_id = thread.get_ident() if nested else object()
+
+        try:
+            yield self._nested_txns_map[cur_thread_id]
+        except KeyError:
             with self.create_transaction(
                     check_error, log_errors, **kwargs) as txn:
-                self._nested_txn = txn
+                self._nested_txns_map[cur_thread_id] = txn
                 try:
                     yield txn
                 finally:
-                    self._nested_txn = None
+                    del self._nested_txns_map[cur_thread_id]
 
     @abc.abstractmethod
     def db_create(self, table, **col_values):
@@ -112,9 +127,18 @@ class API(object):
         :param col_values: The columns and their associated values
                            to be set after create
         :type col_values:  Dictionary of columns id's and values
-        :returns:          :class:`Command` with no result
+        :returns:          :class:`Command` with uuid result
         """
 
+    def db_create_row(self, table, **col_values):
+        """Create a command to create new record
+
+        Identical to db_create, but returns a RowView result
+        :returns: :class:`Command` with RowView result
+        """
+        # vif_plug_ovs has a copy of impl_vsctl that doesn't implement this
+        raise NotImplementedError
+
     @abc.abstractmethod
     def db_destroy(self, table, record):
         """Create a command to destroy a record
